// 4x4 Magic Square Calculator by Daniel Marschall
// Version 5

#include <stdio.h>
#include <stdbool.h>
#include <time.h>
#include <gsl/gsl_permutation.h>
     
const char * MERGE_FILE = "Complete.txt";
const char * RES_LINE = "4x4 (%d) = %2d %2d %2d %2d | %2d %2d %2d %2d | %2d %2d %2d %2d | %2d %2d %2d %2d | %c %c\n";

bool wipe_file(const char * filename) {
	FILE *out = fopen(filename, "w");
	if (out) {
		fclose(out);
		return true;
	} else return false;
}

bool write_result(size_t *data) {
	char buffer[255];

	int sum = data[0]+data[1]+data[2]+data[3];
	
	// R = Diagonale LinksOben-RechtsUnten erfllt
	// L = Diagonale RechtsOben-LinksUnten erfllt
	
	char properties[2];
	properties[0] = (sum == data[0]+data[5]+data[10]+data[15]) ? 'R' : '-';
	properties[1] = (sum == data[12]+data[9]+data[6]+data[3]) ? 'L' : '-';

	FILE *out = fopen(MERGE_FILE, "a");
	if (out) {
		sprintf(buffer, RES_LINE, sum+4, data[0]+1, data[1]+1, data[2]+1, data[3]+1, data[4]+1, data[5]+1, data[6]+1, data[7]+1, data[8]+1, data[9]+1, data[10]+1, data[11]+1, data[12]+1, data[13]+1, data[14]+1, data[15]+1, properties[0], properties[1]);

		int res = fputs(buffer, out);
		fclose(out);

		return res >= 0;
	} else return false;
}

bool begin() {
	char buffer[255];

	FILE *out = fopen(MERGE_FILE, "a");
	if (out) {
		sprintf(buffer, "Beginning!\n");

		int res = fputs(buffer, out);
		fclose(out);

		return res >= 0;
	} else return false;
}

bool finish() {
	char buffer[255];

	FILE *out = fopen(MERGE_FILE, "a");
	if (out) {
		sprintf(buffer, "Finished!\n");

		int res = fputs(buffer, out);
		fclose(out);

		return res >= 0;
	} else return false;
}

int main (void) {
	if ((!wipe_file(MERGE_FILE)) || (!begin())) {
		printf("FATAL ERROR AT INITIALIZE!\n");
		return 1;
	}

	gsl_permutation * p = gsl_permutation_alloc (16);

	gsl_permutation_init (p);

	int last_d = -1;
	int new_time;
	int old_time = -1;
	int time_diff;
	bool estimation_given = false;
     
	do {
		size_t* data = gsl_permutation_data (p);

		if (!estimation_given) {
			if (last_d != data[3]) {
				last_d = data[3];
				new_time = (int) time (NULL);
				if (old_time != -1) {
					time_diff = new_time - old_time;
					printf("Estimated time = %f days\n", (float)16*15*14*13*time_diff/60/60/24);
					estimation_given = true;
				}
				old_time = new_time;
			}
		}

		int sum = data[0]+data[1]+data[2]+data[3];
		if ((sum == data[4]+data[5]+data[6]+data[7]) && (sum == data[8]+data[9]+data[10]+data[11]) && (sum == data[12]+data[13]+data[14]+data[15]) &&
			(sum == data[0]+data[4]+data[8]+data[12]) && (sum == data[1]+data[5]+data[9]+data[13]) && (sum == data[2]+data[6]+data[10]+data[14]) && (sum == data[3]+data[7]+data[11]+data[15])
			/* && (sum == data[0]+data[5]+data[10]+data[15]) */ ) {
			if (!write_result(data)) {
				printf("FATAL ERROR AT WRITING RESULT!\n");
				return 1;
			}
		}
	}
	while (gsl_permutation_next(p) == GSL_SUCCESS);
     
	gsl_permutation_free (p);

	if (!finish()) {
		printf("FATAL ERROR AT FINALIZE!\n");
		return 1;
	}
     
	return 0;
}
